
(* Paramètres et mise en place de la socket serveur *)

let port = 4012
let max_conn = 1000
let bind_addr = Unix.inet_addr_of_string "0.0.0.0"

let socket = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0

let () =
  let bind_addr = Unix.ADDR_INET(bind_addr,port) in
  try
    Unix.setsockopt socket Unix.SO_REUSEADDR true ;
    Unix.bind socket bind_addr ;
    Unix.listen socket max_conn ;
    Printf.printf "Listening on port %d...\n%!" port
  with
    | Unix.Unix_error(Unix.EADDRINUSE, "bind", "") ->
        failwith "port already in use"

let name_of_sockaddr = function
  | Unix.ADDR_UNIX s -> "<unix>"
  | Unix.ADDR_INET (x,p) -> 
      try
        (Unix.gethostbyaddr x).Unix.h_name
      with
        | Not_found -> Unix.string_of_inet_addr x

(* Liste des (socket) clients connectés *)

let clients = ref []

let close_client socket =
  Unix.close socket ;
  clients := List.filter ((<>) socket) !clients

(* Traitement d'un client: on écoute et re-diffuse *)

let handle_client cli_socket =
  let bufsize = 60 in
  let buffer = String.create bufsize in
  let rec run () =
    let j = Unix.read cli_socket buffer 0 bufsize in
      if j<=0 then begin
        Printf.printf
          "Error (%d) when reading from client %d.\n%!"
          j (Obj.magic cli_socket) ;
        close_client cli_socket
      end else begin
        Printf.printf
          "%d sent %d bytes: %S\n%!"
          (Obj.magic cli_socket) j (String.sub buffer 0 j) ;
        List.iter
          (fun client ->
             if client <> cli_socket then
               ignore (Unix.write client buffer 0 j))
          !clients ;
        run ()
      end
  in
    run ()

(* Boucle serveur: pour chaque connection on lance un thread *)

let rec server_loop () =
  let (client,caller) = Unix.accept socket in
  let ip = name_of_sockaddr caller in
    Printf.printf "New client on %s.\n%!" ip ;
    clients := client :: !clients ;
    ignore (Thread.create handle_client client) ;
    server_loop ()

let () = server_loop ()
